//-----------------------------------------------------------------------------
//
//  potentiometer.c
//
//  WFF UDB GenericHID Demonstration Firmware (4_0)
//  A demonstration firmware for the WFF GenericHID Communication Library
//
//  Copyright (c) 2011 Simon Inns
//
//  Permission is hereby granted, free of charge, to any person obtaining a
//  copy of this software and associated documentation files (the "Software"),
//  to deal in the Software without restriction, including without limitation
//  the rights to use, copy, modify, merge, publish, distribute, sublicense,
//  and/or sell copies of the Software, and to permit persons to whom the
//  Software is furnished to do so, subject to the following conditions:
//
//  The above copyright notice and this permission notice shall be included in
//  all copies or substantial portions of the Software.
//
//  THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
//  IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
//  FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
//  AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
//  LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING
//  FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER
//  DEALINGS IN THE SOFTWARE.
//
//  Web:    http://www.waitingforfriday.com
//  Email:  simon.inns@gmail.com
//
//-----------------------------------------------------------------------------

// Global includes
#include <avr/io.h>

// Local includes
#include "potentiometer.h"

// Initialize the potentiometer for reading
void initialisePot(void)
{
	// Configure ADMUX for 5V Vref - Left justified - ADC channel 0
	ADMUX = 0x40; // 01000000
	
	// Configure ADSC for ADC Enabled - /128 pre-scaler
	ADCSRA = 0x87; // 10000111
}

// Read the current value of the potentiometer
uint16_t readPotValue(void)
{
	uint16_t potValue = 0;
	
	 // Clear the previous result
	ADCH = 0x00;
	
	// Set ADSC to start an ADC conversion
	sbi(ADCSRA, 6);

	// Wait for the ADC conversion to complete
	while(ADCSRA & (1 << ADSC));
	
	// Store the result
	potValue = ADC;
	
	return potValue;
}