//-----------------------------------------------------------------------------
//
//  GenericHID.h
//
//  WFF UDB GenericHID Reference Firmware (4_0)
//  A reference firmware for the WFF GenericHID Communication Library
//
//  Copyright (c) 2011 Simon Inns
//
//  Permission is hereby granted, free of charge, to any person obtaining a
//  copy of this software and associated documentation files (the "Software"),
//  to deal in the Software without restriction, including without limitation
//  the rights to use, copy, modify, merge, publish, distribute, sublicense,
//  and/or sell copies of the Software, and to permit persons to whom the
//  Software is furnished to do so, subject to the following conditions:
//
//  The above copyright notice and this permission notice shall be included in
//  all copies or substantial portions of the Software.
//
//  THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
//  IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
//  FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
//  AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
//  LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING
//  FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER
//  DEALINGS IN THE SOFTWARE.
//
//  Web:    http://www.waitingforfriday.com
//  Email:  simon.inns@gmail.com
//
//-----------------------------------------------------------------------------

#ifndef _GENERICHID_H_
#define _GENERICHID_H_

// Global includes
#include <avr/io.h>
#include <avr/wdt.h>
#include <avr/power.h>
#include <avr/interrupt.h>
#include <string.h>

// Local includes
#include "Descriptors.h"
#include "debug.h"

// LUFA includes
#include "LUFA/Version.h"
#include "LUFA/Drivers/Board/LEDs.h"
#include "LUFA/Drivers/USB/USB.h"

// Macros
// LED mask for the library LED driver, to indicate that the USB interface is not ready
#define LEDMASK_USB_NOTREADY      LEDS_LED1

//LED mask for the library LED driver, to indicate that the USB interface is enumerating
#define LEDMASK_USB_ENUMERATING  (LEDS_LED2 | LEDS_LED3)

// LED mask for the library LED driver, to indicate that the USB interface is ready
#define LEDMASK_USB_READY        (LEDS_LED2 | LEDS_LED4)

// LED mask for the library LED driver, to indicate that an error has occurred in the USB interface
#define LEDMASK_USB_ERROR        (LEDS_LED1 | LEDS_LED3)

// Function Prototypes:
void setupHardware(void);

void EVENT_USB_Device_Connect(void);
void EVENT_USB_Device_Disconnect(void);
void EVENT_USB_Device_ConfigurationChanged(void);
void EVENT_USB_Device_ControlRequest(void);
void EVENT_USB_Device_StartOfFrame(void);

#endif

